# PostmanRestapi Python SDK 1.0.0

Welcome to the PostmanRestapi SDK documentation. This guide will help you get started with integrating and using the PostmanRestapi SDK in your project.

## Versions

- API version: `1.0`
- SDK version: `1.0.0`

## About the API

The Postman API enables you to programmatically access data stored in your Postman account.

## Table of Contents

- [Setup & Configuration](#setup--configuration)
  - [Supported Language Versions](#supported-language-versions)
  - [Installation](#installation)
- [Authentication](#authentication)
  - [API Key Authentication](#api-key-authentication)
- [Using Union Types](#using-union-types)
- [Services](#services)
- [Models](#models)
- [License](#license)

## Setup & Configuration

### Supported Language Versions

This SDK is compatible with the following versions: `Python >= 3.7`

### Installation

To get started with the SDK, we recommend installing using `pip`:

```bash
pip install postman-restapi
```

## Authentication

### API Key Authentication

The PostmanRestapi API uses API keys as a form of authentication. An API key is a unique identifier used to authenticate a user, developer, or a program that is calling the API.

#### Setting the API key

When you initialize the SDK, you can set the API key as follows:

```py
PostmanRestapi(
    api_key="YOUR_API_KEY",
    api_key_header="YOUR_API_KEY_HEADER"
)
```

If you need to set or update the API key after initializing the SDK, you can use:

```py
sdk.set_api_key("YOUR_API_KEY", "YOUR_API_KEY_HEADER")
```

## Using Union Types

Union types allow you to specify that a variable can have more than one type. This is particularly useful when a function can accept multiple types of inputs. The Union type hint is used for this purpose.

### Example Function with Union Types

You can call service method with an instance of `TypeA`, `TypeB`, or a dictionary that can be converted to an instance of either type.

```python
# Model Definition
ParamType = Union[TypeA, TypeB]

# Service Method
def service_method(param: ParamType):
...

## Usage
type_a = TypeA(key="value")
type_b = TypeB(key="value")

sdk.service.service_method(type_a)
sdk.service.service_method(type_b)
sdk.service.service_method({"key": "value"})
```

You cannot create an instance of a `Union` type itself. Instead, pass an instance of one of the types in the `Union`, or a dictionary that can be converted to one of those types.

## Services

The SDK provides various services to interact with the API.

<details> 
<summary>Below is a list of all available services with links to their detailed documentation:</summary>

| Name                                                                               |
| :--------------------------------------------------------------------------------- |
| [BillingService](documentation/services/BillingService.md)                         |
| [ApiService](documentation/services/ApiService.md)                                 |
| [TagsService](documentation/services/TagsService.md)                               |
| [AuditLogsService](documentation/services/AuditLogsService.md)                     |
| [CollectionsService](documentation/services/CollectionsService.md)                 |
| [CollectionItemsService](documentation/services/CollectionItemsService.md)         |
| [CollectionFoldersService](documentation/services/CollectionFoldersService.md)     |
| [CollectionRequestsService](documentation/services/CollectionRequestsService.md)   |
| [CollectionResponsesService](documentation/services/CollectionResponsesService.md) |
| [SecretScannerService](documentation/services/SecretScannerService.md)             |
| [EnvironmentsService](documentation/services/EnvironmentsService.md)               |
| [ImportService](documentation/services/ImportService.md)                           |
| [UserService](documentation/services/UserService.md)                               |
| [MocksService](documentation/services/MocksService.md)                             |
| [MonitorsService](documentation/services/MonitorsService.md)                       |
| [PrivateApiNetworkService](documentation/services/PrivateApiNetworkService.md)     |
| [PullRequestsService](documentation/services/PullRequestsService.md)               |
| [ApiSecurityService](documentation/services/ApiSecurityService.md)                 |
| [ScimService](documentation/services/ScimService.md)                               |
| [WebhooksService](documentation/services/WebhooksService.md)                       |
| [WorkspacesService](documentation/services/WorkspacesService.md)                   |

</details>

## Models

The SDK includes several models that represent the data structures used in API requests and responses. These models help in organizing and managing the data efficiently.

<details> 
<summary>Below is a list of all available models with links to their detailed documentation:</summary>

| Name                                                                                                                                  | Description                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| :------------------------------------------------------------------------------------------------------------------------------------ | :----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| [InvoicesAccountInfo](documentation/models/InvoicesAccountInfo.md)                                                                    | Information about the account.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [GetAccountInvoices](documentation/models/GetAccountInvoices.md)                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetAccountInvoicesStatus](documentation/models/GetAccountInvoicesStatus.md)                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetApis](documentation/models/GetApis.md)                                                                                            | Information about the API schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [Accept](documentation/models/Accept.md)                                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateApiRequest](documentation/models/CreateApiRequest.md)                                                                          | Information about the API.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [CreateApiOkResponse](documentation/models/CreateApiOkResponse.md)                                                                    | The API's base data schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [GetApi](documentation/models/GetApi.md)                                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetApiInclude](documentation/models/GetApiInclude.md)                                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateApiRequest](documentation/models/UpdateApiRequest.md)                                                                          | Information about the API.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [UpdateApiOkResponse](documentation/models/UpdateApiOkResponse.md)                                                                    | Information about the API.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [AddApiCollectionRequest](documentation/models/AddApiCollectionRequest.md)                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [AddApiCollectionOkResponse](documentation/models/AddApiCollectionOkResponse.md)                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [SyncCollectionWithSchema](documentation/models/SyncCollectionWithSchema.md)                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CommentResponse](documentation/models/CommentResponse.md)                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CommentCreateUpdate](documentation/models/CommentCreateUpdate.md)                                                                    | Information about the comment.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [CommentCreatedUpdated](documentation/models/CommentCreatedUpdated.md)                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateApiSchemaRequest](documentation/models/CreateApiSchemaRequest.md)                                                              | Information about the API schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [CreateApiSchemaOkResponse](documentation/models/CreateApiSchemaOkResponse.md)                                                        | Information about the created API schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [GetApiSchema](documentation/models/GetApiSchema.md)                                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetApiSchemaFiles](documentation/models/GetApiSchemaFiles.md)                                                                        | Information about the schema files and its meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             |
| [GetApiSchemaFileContents](documentation/models/GetApiSchemaFileContents.md)                                                          | Information about the schema file.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [CreateUpdateApiSchemaFileRequest](documentation/models/CreateUpdateApiSchemaFileRequest.md)                                          | Information about schema file.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [CreateUpdateApiSchemaFileOkResponse](documentation/models/CreateUpdateApiSchemaFileOkResponse.md)                                    | Information about the schema file.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [GetStatusOfAnAsyncTask](documentation/models/GetStatusOfAnAsyncTask.md)                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetApiVersions](documentation/models/GetApiVersions.md)                                                                              | Information about the API's versions.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [CreateApiVersionRequest](documentation/models/CreateApiVersionRequest.md)                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateApiVersionAcceptedResponse](documentation/models/CreateApiVersionAcceptedResponse.md)                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetApiVersion](documentation/models/GetApiVersion.md)                                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateApiVersionRequest](documentation/models/UpdateApiVersionRequest.md)                                                            | Information about the API version.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [UpdateApiVersionOkResponse](documentation/models/UpdateApiVersionOkResponse.md)                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [TagGetPut](documentation/models/TagGetPut.md)                                                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [TagUpdateTags](documentation/models/TagUpdateTags.md)                                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetTaggedEntities](documentation/models/GetTaggedEntities.md)                                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [AscDescDefaultDesc](documentation/models/AscDescDefaultDesc.md)                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetTaggedEntitiesEntityType](documentation/models/GetTaggedEntitiesEntityType.md)                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetAuditLogs](documentation/models/GetAuditLogs.md)                                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetCollections](documentation/models/GetCollections.md)                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CollectionsCreateCollectionRequest](documentation/models/CollectionsCreateCollectionRequest.md)                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateCollectionOkResponse](documentation/models/CreateCollectionOkResponse.md)                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateCollectionForkRequest](documentation/models/CreateCollectionForkRequest.md)                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateCollectionForkOkResponse](documentation/models/CreateCollectionForkOkResponse.md)                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MergeCollectionForkRequest](documentation/models/MergeCollectionForkRequest.md)                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MergeCollectionForkOkResponse](documentation/models/MergeCollectionForkOkResponse.md)                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetCollection](documentation/models/GetCollection.md)                                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetCollectionModel](documentation/models/GetCollectionModel.md)                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PutCollectionRequest](documentation/models/PutCollectionRequest.md)                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PutCollectionOkResponse](documentation/models/PutCollectionOkResponse.md)                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PatchCollectionRequest](documentation/models/PatchCollectionRequest.md)                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PatchCollectionOkResponse](documentation/models/PatchCollectionOkResponse.md)                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteCollection](documentation/models/DeleteCollection.md)                                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetCollectionsForkedByUser](documentation/models/GetCollectionsForkedByUser.md)                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [AscDesc](documentation/models/AscDesc.md)                                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetCollectionForks](documentation/models/GetCollectionForks.md)                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PullCollectionChanges](documentation/models/PullCollectionChanges.md)                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetCollectionPullRequests](documentation/models/GetCollectionPullRequests.md)                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PullRequestCreate](documentation/models/PullRequestCreate.md)                                                                        | Information about the pull request.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| [PullRequestCreated](documentation/models/PullRequestCreated.md)                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetCollectionRoles](documentation/models/GetCollectionRoles.md)                                                                      | Information about the collection's roles.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [UpdateCollectionRoles](documentation/models/UpdateCollectionRoles.md)                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetSourceCollectionStatus](documentation/models/GetSourceCollectionStatus.md)                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [TransformCollectionToOpenApi](documentation/models/TransformCollectionToOpenApi.md)                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [Format](documentation/models/Format.md)                                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [TransferCollectionItems](documentation/models/TransferCollectionItems.md)                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [TransferCollectionItems200Error](documentation/models/TransferCollectionItems200Error.md)                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateCollectionFolder](documentation/models/CreateCollectionFolder.md)                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateCollectionRequestOkResponse](documentation/models/CreateCollectionRequestOkResponse.md)                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateCollectionResponse](documentation/models/CreateCollectionResponse.md)                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetCollectionFolder](documentation/models/GetCollectionFolder.md)                                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateCollectionFolder](documentation/models/UpdateCollectionFolder.md)                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteCollectionFolder](documentation/models/DeleteCollectionFolder.md)                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetCollectionRequest](documentation/models/GetCollectionRequest.md)                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateCollectionRequest](documentation/models/UpdateCollectionRequest.md)                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteCollectionRequest](documentation/models/DeleteCollectionRequest.md)                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetCollectionResponse](documentation/models/GetCollectionResponse.md)                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateCollectionResponse](documentation/models/UpdateCollectionResponse.md)                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteCollectionResponse](documentation/models/DeleteCollectionResponse.md)                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DetectedSecretsQueriesRequest](documentation/models/DetectedSecretsQueriesRequest.md)                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DetectedSecretsQueriesOkResponse](documentation/models/DetectedSecretsQueriesOkResponse.md)                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateDetectedSecretResolutionsRequest](documentation/models/UpdateDetectedSecretResolutionsRequest.md)                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateDetectedSecretResolutionsOkResponse](documentation/models/UpdateDetectedSecretResolutionsOkResponse.md)                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetSecretsLocations](documentation/models/GetSecretsLocations.md)                                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetSecretTypes](documentation/models/GetSecretTypes.md)                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetEnvironments](documentation/models/GetEnvironments.md)                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateEnvironmentRequest](documentation/models/CreateEnvironmentRequest.md)                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateEnvironmentOkResponse](documentation/models/CreateEnvironmentOkResponse.md)                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetEnvironment](documentation/models/GetEnvironment.md)                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateEnvironmentRequest](documentation/models/UpdateEnvironmentRequest.md)                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateEnvironmentOkResponse](documentation/models/UpdateEnvironmentOkResponse.md)                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteEnvironment](documentation/models/DeleteEnvironment.md)                                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetEnvironmentForks](documentation/models/GetEnvironmentForks.md)                                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetEnvironmentForksSort](documentation/models/GetEnvironmentForksSort.md)                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ForkEnvironmentRequest](documentation/models/ForkEnvironmentRequest.md)                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ForkEnvironmentOkResponse](documentation/models/ForkEnvironmentOkResponse.md)                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MergeEnvironmentForkRequest](documentation/models/MergeEnvironmentForkRequest.md)                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MergeEnvironmentForkOkResponse](documentation/models/MergeEnvironmentForkOkResponse.md)                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PullEnvironmentRequest](documentation/models/PullEnvironmentRequest.md)                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PullEnvironmentOkResponse](documentation/models/PullEnvironmentOkResponse.md)                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ImportExportFile](documentation/models/ImportExportFile.md)                                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ImportOpenApiDefinitionOkResponse](documentation/models/ImportOpenApiDefinitionOkResponse.md)                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetAuthenticatedUser](documentation/models/GetAuthenticatedUser.md)                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetMocks](documentation/models/GetMocks.md)                                                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateMock](documentation/models/CreateMock.md)                                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MockCreateUpdate](documentation/models/MockCreateUpdate.md)                                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetMock](documentation/models/GetMock.md)                                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateMock](documentation/models/UpdateMock.md)                                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteMock](documentation/models/DeleteMock.md)                                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetMockCallLogs](documentation/models/GetMockCallLogs.md)                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetMockCallLogsSort](documentation/models/GetMockCallLogsSort.md)                                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PublishMock](documentation/models/PublishMock.md)                                                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UnpublishMock](documentation/models/UnpublishMock.md)                                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetMockServerResponses](documentation/models/GetMockServerResponses.md)                                                              | Information about the server response.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [CreateMockServerResponse](documentation/models/CreateMockServerResponse.md)                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateMockServerResponse](documentation/models/UpdateMockServerResponse.md)                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteMockServerResponse](documentation/models/DeleteMockServerResponse.md)                                                          | Information about the deleted server response.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [GetMonitors](documentation/models/GetMonitors.md)                                                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateMonitorRequest](documentation/models/CreateMonitorRequest.md)                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateMonitorOkResponse](documentation/models/CreateMonitorOkResponse.md)                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetMonitor](documentation/models/GetMonitor.md)                                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateMonitorRequest](documentation/models/UpdateMonitorRequest.md)                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateMonitorOkResponse](documentation/models/UpdateMonitorOkResponse.md)                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteMonitor](documentation/models/DeleteMonitor.md)                                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [RunMonitor](documentation/models/RunMonitor.md)                                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetPanElementsAndFolders](documentation/models/GetPanElementsAndFolders.md)                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetAllElementsAndFoldersSort](documentation/models/GetAllElementsAndFoldersSort.md)                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetAllElementsAndFoldersType](documentation/models/GetAllElementsAndFoldersType.md)                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PostPanElementOrFolderRequest](documentation/models/PostPanElementOrFolderRequest.md)                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdatePanElementOrFolderRequest](documentation/models/UpdatePanElementOrFolderRequest.md)                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdatePanElementOrFolderElementType](documentation/models/UpdatePanElementOrFolderElementType.md)                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeletePanElementOrFolder](documentation/models/DeletePanElementOrFolder.md)                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetAllPanAddElementRequests](documentation/models/GetAllPanAddElementRequests.md)                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetAllPanAddElementRequestsStatus](documentation/models/GetAllPanAddElementRequestsStatus.md)                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PrivateApiNetworkRespondPanElementAddRequestRequest_1](documentation/models/PrivateApiNetworkRespondPanElementAddRequestRequest1.md) |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [RespondPanElementAddRequestOkResponse](documentation/models/RespondPanElementAddRequestOkResponse.md)                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PullRequestGet](documentation/models/PullRequestGet.md)                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PullRequestUpdate](documentation/models/PullRequestUpdate.md)                                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PullRequestUpdated](documentation/models/PullRequestUpdated.md)                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [SchemaSecurityValidationRequest](documentation/models/SchemaSecurityValidationRequest.md)                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [SchemaSecurityValidationOkResponse](documentation/models/SchemaSecurityValidationOkResponse.md)                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetScimGroupResources](documentation/models/GetScimGroupResources.md)                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateScimGroupRequest](documentation/models/CreateScimGroupRequest.md)                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateScimGroupCreatedResponse](documentation/models/CreateScimGroupCreatedResponse.md)                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetScimGroupResource](documentation/models/GetScimGroupResource.md)                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ScimUpdateGroupRequest](documentation/models/ScimUpdateGroupRequest.md)                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ScimUpdateGroupOkResponse](documentation/models/ScimUpdateGroupOkResponse.md)                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetScimResourceTypes](documentation/models/GetScimResourceTypes.md)                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetScimServiceProviderConfig](documentation/models/GetScimServiceProviderConfig.md)                                                  | Information about Postman's SCIM API configurations and supported operations.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [GetScimUserResources](documentation/models/GetScimUserResources.md)                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateScimUserRequest](documentation/models/CreateScimUserRequest.md)                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateScimUserCreatedResponse](documentation/models/CreateScimUserCreatedResponse.md)                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetScimUserResourceOkResponse](documentation/models/GetScimUserResourceOkResponse.md)                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateScimUser](documentation/models/UpdateScimUser.md)                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateScimUserState](documentation/models/UpdateScimUserState.md)                                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateWebhookRequest](documentation/models/CreateWebhookRequest.md)                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateWebhookOkResponse](documentation/models/CreateWebhookOkResponse.md)                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetWorkspaces](documentation/models/GetWorkspaces.md)                                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetWorkspacesType](documentation/models/GetWorkspacesType.md)                                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetWorkspacesInclude](documentation/models/GetWorkspacesInclude.md)                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateWorkspaceRequest](documentation/models/CreateWorkspaceRequest.md)                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateWorkspaceOkResponse](documentation/models/CreateWorkspaceOkResponse.md)                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetWorkspaceRoles](documentation/models/GetWorkspaceRoles.md)                                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetWorkspace](documentation/models/GetWorkspace.md)                                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateWorkspaceRequest](documentation/models/UpdateWorkspaceRequest.md)                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateWorkspaceOkResponse](documentation/models/UpdateWorkspaceOkResponse.md)                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteWorkspace](documentation/models/DeleteWorkspace.md)                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetWorkspaceGlobalVariables](documentation/models/GetWorkspaceGlobalVariables.md)                                                    | Information about the workspace's global variables.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| [UpdateWorkspaceGlobalVariablesRequest](documentation/models/UpdateWorkspaceGlobalVariablesRequest.md)                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateWorkspaceGlobalVariablesOkResponse](documentation/models/UpdateWorkspaceGlobalVariablesOkResponse.md)                          | Information about the workspace's updated global variables.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [UpdateWorkspaceRolesRequest](documentation/models/UpdateWorkspaceRolesRequest.md)                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateWorkspaceRolesOkResponse](documentation/models/UpdateWorkspaceRolesOkResponse.md)                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [SalesChannel](documentation/models/SalesChannel.md)                                                                                  | The sales channel from which the account was created:<br>- `SELF_SERVE` — The user purchased the account plan.<br>- `SALES_SERVE` — The account was purchased through the Postman sales team process.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [Slots](documentation/models/Slots.md)                                                                                                | Information about the team's slots.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| [InvoiceData](documentation/models/InvoiceData.md)                                                                                    | Information about the invoice.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [TotalAmount](documentation/models/TotalAmount.md)                                                                                    | Information about the invoice's total billed amount.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     |
| [Links](documentation/models/Links.md)                                                                                                | A [JSON API spec](https://jsonapi.org/format/#document-links) object containing hypermedia links.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [Web](documentation/models/Web.md)                                                                                                    | An object containing web-based account references.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [GetApisApis](documentation/models/GetApisApis.md)                                                                                    | The API's base data schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [GetApisMeta](documentation/models/GetApisMeta.md)                                                                                    | The response's meta information for paginated results.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [GetApi_1](documentation/models/GetApi1.md)                                                                                           | The API's base data schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [GetApi_2](documentation/models/GetApi2.md)                                                                                           |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GitInfo](documentation/models/GitInfo.md)                                                                                            | Information about the API's Git repository integration.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| [GetApi_2Schemas](documentation/models/GetApi2Schemas.md)                                                                             | Information about the schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [GetApi_2Versions](documentation/models/GetApi2Versions.md)                                                                           | Information about the version.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [GetApi_2Collections](documentation/models/GetApi2Collections.md)                                                                     | Information about the collection.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [AddApiCollection_1](documentation/models/AddApiCollection1.md)                                                                       |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [AddApiCollection_2](documentation/models/AddApiCollection2.md)                                                                       |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [AddApiCollection_3](documentation/models/AddApiCollection3.md)                                                                       |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [AddApiCollection_1Data](documentation/models/AddApiCollection1Data.md)                                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [AddApiCollection_1OperationType](documentation/models/AddApiCollection1OperationType.md)                                             | The `COPY_COLLECTION` method.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [AddApiCollection_2OperationType](documentation/models/AddApiCollection2OperationType.md)                                             | The `GENERATE_FROM_SCHEMA` method.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [AddApiCollection_3Data](documentation/models/AddApiCollection3Data.md)                                                               | Information about the collection's contents, such as requests and responses. For a complete list of values, refer to the [collection.json schema file](https://schema.postman.com/json/collection/v2.1.0/collection.json).                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [AddApiCollection_3OperationType](documentation/models/AddApiCollection3OperationType.md)                                             | The `CREATE_NEW` method.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [DataInfo](documentation/models/DataInfo.md)                                                                                          | Information about the collection.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [InfoSchema](documentation/models/InfoSchema.md)                                                                                      | The collection's schema format.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CommentResponseData](documentation/models/CommentResponseData.md)                                                                    | Information about the comment.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [CommentCreateUpdateTags](documentation/models/CommentCreateUpdateTags.md)                                                            | Information about users tagged in the `body` comment.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [UserName](documentation/models/UserName.md)                                                                                          | An object that contains information about the tagged user. The object's name is the user's Postman username. For example, `@user-postman`.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [UserNameType](documentation/models/UserNameType.md)                                                                                  | The `user` value.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [CommentCreatedUpdatedData](documentation/models/CommentCreatedUpdatedData.md)                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateApiSchemaType_1](documentation/models/CreateApiSchemaType1.md)                                                                 | The schema's type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [CreateApiSchemaFiles_1](documentation/models/CreateApiSchemaFiles1.md)                                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [FilesRoot](documentation/models/FilesRoot.md)                                                                                        | Information about the schema's root file.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [Enabled](documentation/models/Enabled.md)                                                                                            | If true, tag the file as the root file. The root tag is only allowed for protobuf specifications.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [CreateApiSchemaType_2](documentation/models/CreateApiSchemaType2.md)                                                                 | The schema's type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [CreateApiSchemaFiles_2](documentation/models/CreateApiSchemaFiles2.md)                                                               | Information about the schema file.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [GetApiSchema_1](documentation/models/GetApiSchema1.md)                                                                               | Information about the schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [GetApiSchema_2](documentation/models/GetApiSchema2.md)                                                                               | Information about the schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [GetApiSchema_1Files](documentation/models/GetApiSchema1Files.md)                                                                     | Information about the schema's files. The response is paginated and limited to one page.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [FilesData](documentation/models/FilesData.md)                                                                                        | Information about the schema file.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [FilesMeta](documentation/models/FilesMeta.md)                                                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetApiSchemaFilesMeta](documentation/models/GetApiSchemaFilesMeta.md)                                                                | The schema's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [GetApiSchemaFilesFiles](documentation/models/GetApiSchemaFilesFiles.md)                                                              | Information about the schema file.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [CreateUpdateApiSchemaFileRoot](documentation/models/CreateUpdateApiSchemaFileRoot.md)                                                | Information about the schema's root file. This tag only applies to protobuf specifications.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [GetStatusOfAnAsyncTaskMeta](documentation/models/GetStatusOfAnAsyncTaskMeta.md)                                                      | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [GetStatusOfAnAsyncTaskStatus](documentation/models/GetStatusOfAnAsyncTaskStatus.md)                                                  | The task's current status.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [Details](documentation/models/Details.md)                                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MetaModel](documentation/models/MetaModel.md)                                                                                        | The model for which the task is performing the operation.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [Action](documentation/models/Action.md)                                                                                              | The task's action.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [Details_1](documentation/models/Details1.md)                                                                                         | Information about the task's resources.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| [Details_2](documentation/models/Details2.md)                                                                                         | Information about the error that occurred during the task's processing.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| [Details_1Resources](documentation/models/Details1Resources.md)                                                                       |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [Error](documentation/models/Error.md)                                                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetApiVersionsMeta](documentation/models/GetApiVersionsMeta.md)                                                                      | The response's meta information for paginated results.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [GetApiVersionsVersions](documentation/models/GetApiVersionsVersions.md)                                                              | Information about the API version.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [CreateApiVersion_1](documentation/models/CreateApiVersion1.md)                                                                       | Information about the API version.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [CreateApiVersion_2](documentation/models/CreateApiVersion2.md)                                                                       | Information about the API version.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [CreateApiVersion_3](documentation/models/CreateApiVersion3.md)                                                                       | Information about the API version.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [CreateApiVersion_1Schemas](documentation/models/CreateApiVersion1Schemas.md)                                                         | Information about the schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [CreateApiVersion_1Collections](documentation/models/CreateApiVersion1Collections.md)                                                 | Information about the collection.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [CreateApiVersion_2Schemas](documentation/models/CreateApiVersion2Schemas.md)                                                         | Information about the schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [CreateApiVersion_2Collections](documentation/models/CreateApiVersion2Collections.md)                                                 | Information about the collection.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [CreateApiVersion_3Schemas](documentation/models/CreateApiVersion3Schemas.md)                                                         | Information about the schema.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [CreateApiVersion_3Collections](documentation/models/CreateApiVersion3Collections.md)                                                 | Information about the collection.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [Version](documentation/models/Version.md)                                                                                            | Information about the API version.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [VersionSchemas](documentation/models/VersionSchemas.md)                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [VersionCollections](documentation/models/VersionCollections.md)                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [TagGetPutTags](documentation/models/TagGetPutTags.md)                                                                                | Information about the tag.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [TagUpdateTagsTags](documentation/models/TagUpdateTagsTags.md)                                                                        | Information about the tag.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [GetTaggedEntitiesData](documentation/models/GetTaggedEntitiesData.md)                                                                | An object containing the paginated elements.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             |
| [GetTaggedEntitiesMeta](documentation/models/GetTaggedEntitiesMeta.md)                                                                | The response's pagination information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [Entities](documentation/models/Entities.md)                                                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [EntitiesEntityType](documentation/models/EntitiesEntityType.md)                                                                      | The type of Postman element.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             |
| [Trails](documentation/models/Trails.md)                                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [TrailsData](documentation/models/TrailsData.md)                                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [Actor](documentation/models/Actor.md)                                                                                                | Information about the user who preformed the audit event.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [DataUser](documentation/models/DataUser.md)                                                                                          | Information about the user.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [DataTeam](documentation/models/DataTeam.md)                                                                                          | The user's team information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             |
| [GetCollectionsCollections](documentation/models/GetCollectionsCollections.md)                                                        | Information about the collection.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [CollectionsFork](documentation/models/CollectionsFork.md)                                                                            | If the collection is [forked](https://learning.postman.com/docs/collaborating-in-postman/version-control/#forking-postman-entities), the fork's information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             |
| [CreateCollectionCollection_1](documentation/models/CreateCollectionCollection1.md)                                                   | For a complete list of values, refer to the [collection.json schema file](https://schema.postman.com/json/collection/v2.1.0/collection.json).                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [CollectionInfo_1](documentation/models/CollectionInfo1.md)                                                                           | An object that contains basic information about the collection. For a complete list of values, refer to the `definitions.info` entry in the [collection.json schema file](https://schema.postman.com/json/collection/v2.1.0/collection.json).                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [CollectionItem](documentation/models/CollectionItem.md)                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateCollectionCollection_2](documentation/models/CreateCollectionCollection2.md)                                                   |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateCollectionForkCollection](documentation/models/CreateCollectionForkCollection.md)                                              | Information about the forked collection.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [CollectionFork](documentation/models/CollectionFork.md)                                                                              | Information about the collection's fork.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [Strategy](documentation/models/Strategy.md)                                                                                          | The fork's merge strategy:<br>- `deleteSource` — Merge the changes into the parent collection. After the merge process is complete, Postman deletes the fork. You must have Editor access to both the parent and forked collections.<br>- `updateSourceWithDestination` — Merge the changes into the parent collection. Any differences in the parent collection are also made to the fork.<br>                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MergeCollectionForkCollection](documentation/models/MergeCollectionForkCollection.md)                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PutCollectionCollection_1](documentation/models/PutCollectionCollection1.md)                                                         | For a complete list of values, refer to the [collection.json schema file](https://schema.postman.com/json/collection/v2.1.0/collection.json).                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [PutCollectionCollection_2](documentation/models/PutCollectionCollection2.md)                                                         |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PatchCollectionCollection_1](documentation/models/PatchCollectionCollection1.md)                                                     |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CollectionInfo_2](documentation/models/CollectionInfo2.md)                                                                           | An object that contains the collection's updated name and description.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [PatchCollectionCollection_2](documentation/models/PatchCollectionCollection2.md)                                                     |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteCollectionCollection](documentation/models/DeleteCollectionCollection.md)                                                      | Information about the deleted collection.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [GetCollectionsForkedByUserData](documentation/models/GetCollectionsForkedByUserData.md)                                              | Information about the forked collection.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [GetCollectionsForkedByUserMeta](documentation/models/GetCollectionsForkedByUserMeta.md)                                              | The response's meta information for paginated results.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [GetCollectionForksData](documentation/models/GetCollectionForksData.md)                                                              | Information about the forked collection.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [GetCollectionForksMeta](documentation/models/GetCollectionForksMeta.md)                                                              | The response's meta information for paginated results.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [PullCollectionChangesCollection](documentation/models/PullCollectionChangesCollection.md)                                            | Information about the updated collection fork.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [GetCollectionPullRequestsData](documentation/models/GetCollectionPullRequestsData.md)                                                | Information about the pull request.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| [DataStatus](documentation/models/DataStatus.md)                                                                                      | The pull request's current status.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [Group](documentation/models/Group.md)                                                                                                | Information about the group role.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [GetCollectionRolesTeam](documentation/models/GetCollectionRolesTeam.md)                                                              | Information about the team role.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [GetCollectionRolesUser](documentation/models/GetCollectionRolesUser.md)                                                              | Information about the user role.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [GroupRole](documentation/models/GroupRole.md)                                                                                        | The role type:<br>- `VIEWER` — Can view, fork, and export collections.<br>- `EDITOR` — Can edit collections directly.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [TeamRole](documentation/models/TeamRole.md)                                                                                          | The role type:<br>- `VIEWER` — Can view, fork, and export collections.<br>- `EDITOR` — Can edit collections directly.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [UserRole](documentation/models/UserRole.md)                                                                                          | The role type:<br>- `VIEWER` — Can view, fork, and export collections.<br>- `EDITOR` — Can edit collections directly.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [UpdateCollectionRolesRoles](documentation/models/UpdateCollectionRolesRoles.md)                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [RolesOp](documentation/models/RolesOp.md)                                                                                            | The operation to perform on the path.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [RolesPath_1](documentation/models/RolesPath1.md)                                                                                     | The resource to perform the action on.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [RolesValue_1](documentation/models/RolesValue1.md)                                                                                   | Information about the updated role.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| [ValueRole_1](documentation/models/ValueRole1.md)                                                                                     | The role type:<br>- `VIEWER` — Can view, fork, and export collections.<br>- `EDITOR` — Can edit collections directly.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [GetSourceCollectionStatusCollection](documentation/models/GetSourceCollectionStatusCollection.md)                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CollectionUid](documentation/models/CollectionUid.md)                                                                                | Information about the forked collection. The object's name is the forked collection's UID (`userId`-`collectionId`).                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     |
| [Mode](documentation/models/Mode.md)                                                                                                  | The transfer operation to perform.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [Target](documentation/models/Target.md)                                                                                              | Information about the item transfer's destination location.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [Location](documentation/models/Location.md)                                                                                          | The transferred items' placement in the target destination:<br>- For `start` or `end` — Do not include the `model` and `id` values.<br>- For `before` or `after` — Include the `model` and `id` properties.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [TargetModel](documentation/models/TargetModel.md)                                                                                    | The collection, folder, or request the items will be transferred to. For response transfers, use the `request` value.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [Position](documentation/models/Position.md)                                                                                          | The transferred item's position within the destination object.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [DeleteCollectionFolderData](documentation/models/DeleteCollectionFolderData.md)                                                      | The folder's information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [DeleteCollectionRequestData](documentation/models/DeleteCollectionRequestData.md)                                                    | The request's information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [DeleteCollectionResponseData](documentation/models/DeleteCollectionResponseData.md)                                                  | The response's information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [Statuses](documentation/models/Statuses.md)                                                                                          | The secret resolution status type:<br>- `FALSE_POSITIVE` — The discovered secret is not an actual secret.<br>- `REVOKED` — The secret is valid, but the user rotated their key to resolve the issue.<br>- `ACCEPTED_RISK` — The Secret Scanner found the secret, but user accepts the risk of publishing it.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [WorkspaceVisiblities](documentation/models/WorkspaceVisiblities.md)                                                                  | The type of visibility setting.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DetectedSecretsQueriesData](documentation/models/DetectedSecretsQueriesData.md)                                                      | Information about the secret finding.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [DetectedSecretsQueriesMeta](documentation/models/DetectedSecretsQueriesMeta.md)                                                      | The response's meta information for paginated results.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [DataResolution](documentation/models/DataResolution.md)                                                                              | The secret's current status:<br>- `ACTIVE` — The secret is active.<br>- `FALSE_POSITIVE` — The discovered secret is not an actual secret.<br>- `REVOKED` — The secret is valid, but the user rotated their key to resolve the issue.<br>- `ACCEPTED_RISK` — The Secret Scanner found the secret, but user accepts the risk of publishing it.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [DataWorkspaceVisibility](documentation/models/DataWorkspaceVisibility.md)                                                            | The workspace's [visibility setting](https://learning.postman.com/docs/collaborating-in-postman/using-workspaces/managing-workspaces/#changing-workspace-visibility).                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [UpdateSecretResolutionsResolution_1](documentation/models/UpdateSecretResolutionsResolution1.md)                                     | The secret's updated resolution status:<br>- `FALSE_POSITIVE` — The discovered secret is not an actual secret.<br>- `REVOKED` — The secret is valid, but the user rotated their key to resolve the issue.<br>- `ACCEPTED_RISK` — The Secret Scanner found the secret, but user accepts the risk of publishing it.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [History](documentation/models/History.md)                                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateSecretResolutionsResolution_2](documentation/models/UpdateSecretResolutionsResolution2.md)                                     | The secret's current resolution status:<br>- `ACTIVE` — The secret is active.<br>- `FALSE_POSITIVE` — The discovered secret is not an actual secret.<br>- `REVOKED` — The secret is valid, but the user rotated their key to resolve the issue.<br>- `ACCEPTED_RISK` — The Secret Scanner found the secret, but user accepts the risk of publishing it.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [HistoryResolution](documentation/models/HistoryResolution.md)                                                                        | The secret's updated resolution status:<br>- `ACTIVE` — The secret is active.<br>- `FALSE_POSITIVE` — The discovered secret is not an actual secret.<br>- `REVOKED` — The secret is valid, but the user rotated their key to resolve the issue.<br>- `ACCEPTED_RISK` — The Secret Scanner found the secret, but user accepts the risk of publishing it.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [GetSecretsLocationsData](documentation/models/GetSecretsLocationsData.md)                                                            | Information about the secret finding locations.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetSecretsLocationsMeta](documentation/models/GetSecretsLocationsMeta.md)                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DataResourceType](documentation/models/DataResourceType.md)                                                                          | The type of resource in which the secret was detected.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [ActivityFeed](documentation/models/ActivityFeed.md)                                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ActivityFeedStatus](documentation/models/ActivityFeedStatus.md)                                                                      | The secret's current resolution status:<br>- `ACTIVE` — The secret is active.<br>- `FALSE_POSITIVE` — The discovered secret is not an actual secret.<br>- `REVOKED` — The secret is valid, but the user rotated their key to resolve the issue.<br>- `ACCEPTED_RISK` — The Secret Scanner found the secret, but user accepts the risk of publishing it.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [GetSecretTypesData](documentation/models/GetSecretTypesData.md)                                                                      | Information about the secret type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [GetSecretTypesMeta](documentation/models/GetSecretTypesMeta.md)                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DataType](documentation/models/DataType.md)                                                                                          | The origin of the secret type:<br>- `DEFAULT` — Supported by default in Postman.<br>- `TEAM_REGEX` — A custom regex added by an Admin or Super Admin user in the **Configure Alerts** section of the [**Team Settings**](https://learning.postman.com/docs/administration/team-settings/) interface.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [GetEnvironmentsEnvironments](documentation/models/GetEnvironmentsEnvironments.md)                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateEnvironmentEnvironment_1](documentation/models/CreateEnvironmentEnvironment1.md)                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [EnvironmentValues_1](documentation/models/EnvironmentValues1.md)                                                                     |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ValuesType_1](documentation/models/ValuesType1.md)                                                                                   | The variable type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [CreateEnvironmentEnvironment_2](documentation/models/CreateEnvironmentEnvironment2.md)                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetEnvironmentEnvironment](documentation/models/GetEnvironmentEnvironment.md)                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [EnvironmentValues_2](documentation/models/EnvironmentValues2.md)                                                                     |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ValuesType_2](documentation/models/ValuesType2.md)                                                                                   | The variable type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [UpdateEnvironmentEnvironment_1](documentation/models/UpdateEnvironmentEnvironment1.md)                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [EnvironmentValues_3](documentation/models/EnvironmentValues3.md)                                                                     |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ValuesType_3](documentation/models/ValuesType3.md)                                                                                   | The variable type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [UpdateEnvironmentEnvironment_2](documentation/models/UpdateEnvironmentEnvironment2.md)                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteEnvironmentEnvironment](documentation/models/DeleteEnvironmentEnvironment.md)                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetEnvironmentForksData](documentation/models/GetEnvironmentForksData.md)                                                            | Information about the forked environment.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [GetEnvironmentForksMeta](documentation/models/GetEnvironmentForksMeta.md)                                                            | The response's meta information for paginated results.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [ForkEnvironmentEnvironment](documentation/models/ForkEnvironmentEnvironment.md)                                                      | Information about the forked environment.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [MergeEnvironmentForkEnvironment](documentation/models/MergeEnvironmentForkEnvironment.md)                                            | Information about the merged environment.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [PullEnvironmentEnvironment](documentation/models/PullEnvironmentEnvironment.md)                                                      | Information about the merged environment.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [JsonSchema](documentation/models/JsonSchema.md)                                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [JsonStringified](documentation/models/JsonStringified.md)                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [JsonSchemaType](documentation/models/JsonSchemaType.md)                                                                              | The OpenAPI definition type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             |
| [JsonStringifiedType](documentation/models/JsonStringifiedType.md)                                                                    | The OpenAPI definition type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             |
| [ImportExportFileType](documentation/models/ImportExportFileType.md)                                                                  | The `file` type value.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [ImportOpenApiDefinitionCollections](documentation/models/ImportOpenApiDefinitionCollections.md)                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetAuthenticatedUserUser](documentation/models/GetAuthenticatedUserUser.md)                                                          | Information about the authenticated user.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [GetAuthenticatedUserOperations](documentation/models/GetAuthenticatedUserOperations.md)                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetMocksMocks](documentation/models/GetMocksMocks.md)                                                                                | Information about the mock servers.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| [MocksConfig](documentation/models/MocksConfig.md)                                                                                    | Information about the mock server's configuration.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [Delay](documentation/models/Delay.md)                                                                                                | Information about the mock server's simulated network delay settings. This returns a null value if there are no configured network delay settings.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [DelayType](documentation/models/DelayType.md)                                                                                        | The type of simulated delay value:<br>- `fixed` — The delay value is a fixed value.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| [Preset](documentation/models/Preset.md)                                                                                              | The simulated fixed network delay value:<br>- `1` — 2G (300 ms).<br>- `2` — 3G (100 ms).<br>The object does not return this value for custom delay values.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [CreateMockMock](documentation/models/CreateMockMock.md)                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MockCreateUpdateMock](documentation/models/MockCreateUpdateMock.md)                                                                  | Information about the mock server.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [GetMockMock](documentation/models/GetMockMock.md)                                                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MockConfig_1](documentation/models/MockConfig1.md)                                                                                   | Information about the mock server's configuration.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [UpdateMockMock](documentation/models/UpdateMockMock.md)                                                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MockConfig_2](documentation/models/MockConfig2.md)                                                                                   | The mock server's configuration settings.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [DeleteMockMock](documentation/models/DeleteMockMock.md)                                                                              | Information about the mock server.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [CallLogs](documentation/models/CallLogs.md)                                                                                          | Information about the mock server's server responses.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [GetMockCallLogsMeta](documentation/models/GetMockCallLogsMeta.md)                                                                    | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [CallLogsRequest](documentation/models/CallLogsRequest.md)                                                                            | The server response's request information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [CallLogsResponse](documentation/models/CallLogsResponse.md)                                                                          | The server response's response information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [RequestHeaders](documentation/models/RequestHeaders.md)                                                                              | The request's headers.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [RequestBody](documentation/models/RequestBody.md)                                                                                    | The request's body information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ResponseHeaders](documentation/models/ResponseHeaders.md)                                                                            | The response's headers.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| [ResponseBody](documentation/models/ResponseBody.md)                                                                                  | The response's body information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [Description](documentation/models/Description.md)                                                                                    | The response header's description information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [PublishMockMock](documentation/models/PublishMockMock.md)                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UnpublishMockMock](documentation/models/UnpublishMockMock.md)                                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateMockServerResponseServerResponse](documentation/models/CreateMockServerResponseServerResponse.md)                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ServerResponseHeaders_1](documentation/models/ServerResponseHeaders1.md)                                                             |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ServerResponseLanguage_1](documentation/models/ServerResponseLanguage1.md)                                                           | The server response's body language type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [UpdateMockServerResponseServerResponse](documentation/models/UpdateMockServerResponseServerResponse.md)                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ServerResponseHeaders_2](documentation/models/ServerResponseHeaders2.md)                                                             | Information about they key-value pair.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [ServerResponseLanguage_2](documentation/models/ServerResponseLanguage2.md)                                                           | The server response's body language type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [DeleteMockServerResponseHeaders](documentation/models/DeleteMockServerResponseHeaders.md)                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteMockServerResponseLanguage](documentation/models/DeleteMockServerResponseLanguage.md)                                          | The server response's body language type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [GetMonitorsMonitors](documentation/models/GetMonitorsMonitors.md)                                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateMonitorMonitor_1](documentation/models/CreateMonitorMonitor1.md)                                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MonitorSchedule_1](documentation/models/MonitorSchedule1.md)                                                                         | Information about the monitor's schedule.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [CreateMonitorMonitor_2](documentation/models/CreateMonitorMonitor2.md)                                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetMonitorMonitor](documentation/models/GetMonitorMonitor.md)                                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [LastRun](documentation/models/LastRun.md)                                                                                            | Information about the monitor's previous run.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [Notifications](documentation/models/Notifications.md)                                                                                | Information about the monitor's notification settings.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [Options](documentation/models/Options.md)                                                                                            | Information about the monitor's option settings.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [MonitorSchedule_2](documentation/models/MonitorSchedule2.md)                                                                         | Information about the monitor's schedule.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [LastRunStats](documentation/models/LastRunStats.md)                                                                                  | Information about the monitor's stats.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [StatsAssertions_1](documentation/models/StatsAssertions1.md)                                                                         | Information about the monitor's assertions.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [StatsRequests_1](documentation/models/StatsRequests1.md)                                                                             | Information about the monitor's requests.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [OnError](documentation/models/OnError.md)                                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [OnFailure](documentation/models/OnFailure.md)                                                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdateMonitorMonitor_1](documentation/models/UpdateMonitorMonitor1.md)                                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MonitorSchedule_3](documentation/models/MonitorSchedule3.md)                                                                         | Information about the monitor's schedule.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [UpdateMonitorMonitor_2](documentation/models/UpdateMonitorMonitor2.md)                                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeleteMonitorMonitor](documentation/models/DeleteMonitorMonitor.md)                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [Run](documentation/models/Run.md)                                                                                                    | Information about the monitor run.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [RunInfo](documentation/models/RunInfo.md)                                                                                            | Information about the monitor.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [RunStats](documentation/models/RunStats.md)                                                                                          | Information about the monitor run's stats.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [Executions](documentation/models/Executions.md)                                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [StatsAssertions_2](documentation/models/StatsAssertions2.md)                                                                         | The monitor run's assertions stats.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| [StatsRequests_2](documentation/models/StatsRequests2.md)                                                                             | The monitor run's request stats.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [ExecutionsItem](documentation/models/ExecutionsItem.md)                                                                              | Information about the executed item.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     |
| [ExecutionsRequest](documentation/models/ExecutionsRequest.md)                                                                        | Information about the monitor run's requests.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [ExecutionsResponse](documentation/models/ExecutionsResponse.md)                                                                      | Information about the monitor run's response.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [Elements](documentation/models/Elements.md)                                                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [Folders](documentation/models/Folders.md)                                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetPanElementsAndFoldersMeta](documentation/models/GetPanElementsAndFoldersMeta.md)                                                  | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [PanCreateApi](documentation/models/PanCreateApi.md)                                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PanCreateCollection](documentation/models/PanCreateCollection.md)                                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PanCreateWorkspace](documentation/models/PanCreateWorkspace.md)                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PanCreateFolder](documentation/models/PanCreateFolder.md)                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PanCreateApiApi](documentation/models/PanCreateApiApi.md)                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PanCreateCollectionCollection](documentation/models/PanCreateCollectionCollection.md)                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PanCreateWorkspaceWorkspace](documentation/models/PanCreateWorkspaceWorkspace.md)                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PanCreateFolderFolder](documentation/models/PanCreateFolderFolder.md)                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [PanElementCreated](documentation/models/PanElementCreated.md)                                                                        | Information about the Private API Network element.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [PanFolderCreated](documentation/models/PanFolderCreated.md)                                                                          | Information about the Private API Network folder.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [UpdatePanApi](documentation/models/UpdatePanApi.md)                                                                                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdatePanCollection](documentation/models/UpdatePanCollection.md)                                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdatePanWorkspace](documentation/models/UpdatePanWorkspace.md)                                                                      |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdatePanFolder](documentation/models/UpdatePanFolder.md)                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdatePanApiApi](documentation/models/UpdatePanApiApi.md)                                                                            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdatePanCollectionCollection](documentation/models/UpdatePanCollectionCollection.md)                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CollectionEnvironments](documentation/models/CollectionEnvironments.md)                                                              | The collection's updated environments.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [UpdatePanWorkspaceWorkspace](documentation/models/UpdatePanWorkspaceWorkspace.md)                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [UpdatePanFolderFolder](documentation/models/UpdatePanFolderFolder.md)                                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DeletePanElementOrFolderElementType](documentation/models/DeletePanElementOrFolderElementType.md)                                    | The Private API Network element type. The name of the object is the element `api`, `collection`, `workspace`, or `folder` type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetAllPanAddElementRequestsRequests](documentation/models/GetAllPanAddElementRequestsRequests.md)                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetAllPanAddElementRequestsMeta](documentation/models/GetAllPanAddElementRequestsMeta.md)                                            | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [RequestsStatus](documentation/models/RequestsStatus.md)                                                                              | The request's status.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [RequestsElement](documentation/models/RequestsElement.md)                                                                            | Information about the requested element.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [RequestsResponse](documentation/models/RequestsResponse.md)                                                                          | Information about the response to the request. This object only returns when the network manager denied a request with a message.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [ElementType_1](documentation/models/ElementType1.md)                                                                                 | The element type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [RespondPanElementAddRequestResponse](documentation/models/RespondPanElementAddRequestResponse.md)                                    | The response to the user's request.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| [RespondPanElementAddRequestStatus](documentation/models/RespondPanElementAddRequestStatus.md)                                        | The request's status.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [RespondPanElementAddRequestRequest_2](documentation/models/RespondPanElementAddRequestRequest2.md)                                   |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [RequestResponse](documentation/models/RequestResponse.md)                                                                            | Information about the response to the element's request. This object only returns when the network manager denied a request with a message.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [RequestElement](documentation/models/RequestElement.md)                                                                              | Information about the requested element.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [RequestStatus](documentation/models/RequestStatus.md)                                                                                | The request's status.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [ElementType_2](documentation/models/ElementType2.md)                                                                                 | The element type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [Source](documentation/models/Source.md)                                                                                              | Information about the pull request's source (parent) element.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [Destination](documentation/models/Destination.md)                                                                                    | Information about the pull request destination element.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| [Merge](documentation/models/Merge.md)                                                                                                | Information about the current progress of the pull request's merge.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| [Reviewers](documentation/models/Reviewers.md)                                                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [MergeStatus](documentation/models/MergeStatus.md)                                                                                    | The pull request's current merge status:<br>- `inactive` — There is no merge in progress.<br>- `inprogress` — The pull request is currently merging.<br>- `failed` — The pull request's merge failed.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [PullRequestUpdatedStatus](documentation/models/PullRequestUpdatedStatus.md)                                                          | The pull request's status.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [SchemaSecurityValidationSchema](documentation/models/SchemaSecurityValidationSchema.md)                                              |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [SchemaLanguage](documentation/models/SchemaLanguage.md)                                                                              | The definition format.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [SchemaType](documentation/models/SchemaType.md)                                                                                      | The definition type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     |
| [ScimGroupResource](documentation/models/ScimGroupResource.md)                                                                        | The SCIM group resource object.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [ScimGroupResourceMembers](documentation/models/ScimGroupResourceMembers.md)                                                          | Information about the group's members.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [ScimGroupResourceMeta](documentation/models/ScimGroupResourceMeta.md)                                                                | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [CreateScimGroupMembers_1](documentation/models/CreateScimGroupMembers1.md)                                                           | An object containing the SCIM users to assign to the group.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [CreateScimGroupMembers_2](documentation/models/CreateScimGroupMembers2.md)                                                           | An object containing the group's assigned SCIM members.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| [CreateScimGroupMeta](documentation/models/CreateScimGroupMeta.md)                                                                    | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [GetScimGroupResourceMembers](documentation/models/GetScimGroupResourceMembers.md)                                                    |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [GetScimGroupResourceMeta](documentation/models/GetScimGroupResourceMeta.md)                                                          | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [ScimUpdateGroupOperations](documentation/models/ScimUpdateGroupOperations.md)                                                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [OperationsOp_1](documentation/models/OperationsOp1.md)                                                                               | The operation to perform.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [OperationsValue_1](documentation/models/OperationsValue1.md)                                                                         | The performed operation's value.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [ScimUpdateGroupMeta](documentation/models/ScimUpdateGroupMeta.md)                                                                    | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [SchemaExtensions](documentation/models/SchemaExtensions.md)                                                                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [Patch](documentation/models/Patch.md)                                                                                                | Information about patch configuration.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [Bulk](documentation/models/Bulk.md)                                                                                                  | Information about bulk configuration.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [Filter](documentation/models/Filter.md)                                                                                              | Information about the filter configuration.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| [ChangePassword](documentation/models/ChangePassword.md)                                                                              | Information about the change password configuration.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     |
| [GetScimServiceProviderConfigSort](documentation/models/GetScimServiceProviderConfigSort.md)                                          | Information about the sort configuration.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [AuthenticationSchemes](documentation/models/AuthenticationSchemes.md)                                                                | Information about the scheme.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [Etag](documentation/models/Etag.md)                                                                                                  | Information about the [entity tag](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/ETag) (Etag) HTTP response header configuration.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [GetScimServiceProviderConfigMeta](documentation/models/GetScimServiceProviderConfigMeta.md)                                          | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [Resources](documentation/models/Resources.md)                                                                                        | The SCIM user resource object.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [ScimUserResourceName_1](documentation/models/ScimUserResourceName1.md)                                                               | Information about the Postman team member.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [ScimUserResourceMeta_1](documentation/models/ScimUserResourceMeta1.md)                                                               | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [CreateScimUserName_1](documentation/models/CreateScimUserName1.md)                                                                   | Information about the user's name.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [CreateScimUserName_2](documentation/models/CreateScimUserName2.md)                                                                   |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateScimUserMeta](documentation/models/CreateScimUserMeta.md)                                                                      | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [MetaResourceType](documentation/models/MetaResourceType.md)                                                                          | The SCIM resource type.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| [ScimUserResourceName_2](documentation/models/ScimUserResourceName2.md)                                                               | Information about the Postman team member.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [ScimUserResourceMeta_2](documentation/models/ScimUserResourceMeta2.md)                                                               | The response's non-standard meta information.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [UpdateScimUserName](documentation/models/UpdateScimUserName.md)                                                                      | Information about the user's name.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [UpdateScimUserStateOperations](documentation/models/UpdateScimUserStateOperations.md)                                                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [OperationsOp_2](documentation/models/OperationsOp2.md)                                                                               | The operation to perform.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| [OperationsValue_2](documentation/models/OperationsValue2.md)                                                                         | The performed operation's value.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [CreateWebhookWebhook_1](documentation/models/CreateWebhookWebhook1.md)                                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [CreateWebhookWebhook_2](documentation/models/CreateWebhookWebhook2.md)                                                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [Workspaces](documentation/models/Workspaces.md)                                                                                      | Information about the workspace.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [WorkspacesType](documentation/models/WorkspacesType.md)                                                                              | The type of workspace.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [WorkspacesVisibility](documentation/models/WorkspacesVisibility.md)                                                                  | The workspace's visibility. [Visibility](https://learning.postman.com/docs/collaborating-in-postman/using-workspaces/managing-workspaces/#changing-workspace-visibility) determines who can access the workspace:<br>- `personal` — Only you can access the workspace.<br>- `team` — All team members can access the workspace.<br>- `private` — Only invited team members can access the workspace ([**Professional** and **Enterprise** plans only](https://www.postman.com/pricing)).<br>- `public` — Everyone can access the workspace.<br>- `partner` — Only invited team members and [partners](https://learning.postman.com/docs/collaborating-in-postman/using-workspaces/partner-workspaces/) can access the workspace ([**Professional** and **Enterprise** plans only](https://www.postman.com/pricing)).<br> |
| [CreateWorkspaceWorkspace_1](documentation/models/CreateWorkspaceWorkspace1.md)                                                       | Information about the workspace.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [WorkspaceType_1](documentation/models/WorkspaceType1.md)                                                                             | The type of workspace:<br>- `personal`<br>- `private` — Private workspaces are available on Postman [**Professional** and **Enterprise** plans](https://www.postman.com/pricing).<br>- `public`<br>- `team`<br>- `partner` — [Partner Workspaces](https://learning.postman.com/docs/collaborating-in-postman/using-workspaces/partner-workspaces/) are available on Postman [**Professional** and **Enterprise** plans](https://www.postman.com/pricing)).<br>                                                                                                                                                                                                                                                                                                                                                           |
| [CreateWorkspaceWorkspace_2](documentation/models/CreateWorkspaceWorkspace2.md)                                                       | Information about the created workspace.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [GetWorkspaceRolesRoles](documentation/models/GetWorkspaceRolesRoles.md)                                                              | Information about the workspace's [user roles](https://learning.postman.com/docs/collaborating-in-postman/roles-and-permissions/#team-roles).                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            |
| [RolesUser](documentation/models/RolesUser.md)                                                                                        | Information about the user role.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [Usergroup](documentation/models/Usergroup.md)                                                                                        | Information about the user group in the workspace.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [GetWorkspaceWorkspace](documentation/models/GetWorkspaceWorkspace.md)                                                                | Information about the workspace.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| [WorkspaceType_2](documentation/models/WorkspaceType2.md)                                                                             | The type of workspace.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [WorkspaceVisibility](documentation/models/WorkspaceVisibility.md)                                                                    | The workspace's visibility. [Visibility](https://learning.postman.com/docs/collaborating-in-postman/using-workspaces/managing-workspaces/#changing-workspace-visibility) determines who can access the workspace:<br>- `personal` — Only you can access the workspace.<br>- `team` — All team members can access the workspace.<br>- `private` — Only invited team members can access the workspace ([**Professional** and **Enterprise** plans only](https://www.postman.com/pricing)).<br>- `public` — Everyone can access the workspace.<br>- `partner` — Only invited team members and [partners](https://learning.postman.com/docs/collaborating-in-postman/using-workspaces/partner-workspaces/) can access the workspace ([**Professional** and **Enterprise** plans only](https://www.postman.com/pricing)).<br> |
| [WorkspaceCollections](documentation/models/WorkspaceCollections.md)                                                                  | Information about the collection.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        |
| [WorkspaceEnvironments](documentation/models/WorkspaceEnvironments.md)                                                                | Information about the environment.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [WorkspaceMocks](documentation/models/WorkspaceMocks.md)                                                                              | Information about the mock server.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| [WorkspaceMonitors](documentation/models/WorkspaceMonitors.md)                                                                        | Information about the monitor.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| [WorkspaceApis](documentation/models/WorkspaceApis.md)                                                                                | Information about the API.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| [UpdateWorkspaceWorkspace_1](documentation/models/UpdateWorkspaceWorkspace1.md)                                                       |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [WorkspaceType_3](documentation/models/WorkspaceType3.md)                                                                             | The new workspace visibility [type](https://learning.postman.com/docs/collaborating-in-postman/using-workspaces/managing-workspaces/#changing-workspace-visibility). This property does not support the following workspace visibility changes:<br>- `private` to `public`, `public` to `private`, and `private` to `personal` for Free and Basic [plans](https://www.postman.com/pricing/).<br>- `public` to `personal` for team users.<br>                                                                                                                                                                                                                                                                                                                                                                             |
| [UpdateWorkspaceWorkspace_2](documentation/models/UpdateWorkspaceWorkspace2.md)                                                       | Information about the updated workspace.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [DeleteWorkspaceWorkspace](documentation/models/DeleteWorkspaceWorkspace.md)                                                          | Information about the deleted workspace.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| [GlobalVariable](documentation/models/GlobalVariable.md)                                                                              | Information about the global variable.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [GlobalVariableType](documentation/models/GlobalVariableType.md)                                                                      | The [type](https://learning.postman.com/docs/sending-requests/variables/#variable-types) of variable.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| [UpdateWorkspaceRolesRoles_1](documentation/models/UpdateWorkspaceRolesRoles1.md)                                                     |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [RolesPath_2](documentation/models/RolesPath2.md)                                                                                     | The resource to perform the action on.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [RolesValue_2](documentation/models/RolesValue2.md)                                                                                   | Information about the updated user or user group role.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| [ValueRole_2](documentation/models/ValueRole2.md)                                                                                     | The workspace role's ID:<br>- `1` — Viewer. Can view, fork, and export workspace resources.<br>- `2` — Editor. Can create and edit workspace resources.<br>- `3` — Admin. Can manage workspace details and members.<br>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| [UpdateWorkspaceRolesRoles_2](documentation/models/UpdateWorkspaceRolesRoles2.md)                                                     |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| [DisplayName](documentation/models/DisplayName.md)                                                                                    | The role's display name.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |

</details>

## License

This SDK is licensed under the MIT License.

See the [LICENSE](LICENSE) file for more details.

<!-- This file was generated by liblab | https://liblab.com/ -->
